unit CreatedAtRuntime_;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, VidGrab, ExtCtrls, Menus;

type
  TForm1 = class(TForm)
    Image1: TImage;
    GroupBox1: TGroupBox;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    CheckBox1: TCheckBox;
    ComboBox1: TComboBox;
    ComboBox2: TComboBox;
    ComboBox3: TComboBox;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ComboBox2Change(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure ComboBox3Change(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
  private
    VideoGrabber: TVideoGrabber;
    VideoGrabberCreated: Boolean;
  public
    procedure ChangeVideoDevice (Sender: TObject);
    procedure OnFrameCaptureCompleted(Sender: TObject; const FrameData: TFrameData;
              DestType: TFrameCaptureDest; FileName: String; Success: Boolean);
    procedure OnResizeVideo(Sender: TObject; SourceWidth,
              SourceHeight: Integer);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
begin
   VideoGrabberCreated := False;
   ComboBox1.Items.Text := VideoDevices;
   ComboBox1.ItemIndex := 0;
end;
//------------------------------------------------------------------------------

procedure TForm1.Button1Click(Sender: TObject);
begin
   if not assigned (VideoGrabber) then begin

      VideoGrabber := TVideoGrabber.Create (Self);
      VideoGrabber.Visible := CheckBox1.Checked;
      VideoGrabber.AutoRefreshPreview := true;

      VideoGrabber.OnChangeVideoDevice := ChangeVideoDevice;
      VideoGrabber.OnFrameCaptureCompleted := OnFrameCaptureCompleted;
      VideoGrabber.OnResizeVideo := OnResizeVideo;

      VideoGrabber.VideoDevice := ComboBox1.ItemIndex;

      VideoGrabber.Left := 8;
      VideoGrabber.Top := 55;

      VideoGrabber.Parent := Self;

      VideoGrabber.StartPreview;
      VideoGrabberCreated := True;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.Button2Click(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.CaptureFrameTo (fc_TBitmap);
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.Button3Click(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.Free;
      VideoGrabberCreated := False;
      VideoGrabber := nil;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.ChangeVideoDevice (Sender: TObject);
begin
   Image1.Picture.Assign (nil);

   ComboBox2.Clear;
   ComboBox2.Items.Text := VideoGrabber.VideoSizes;
   ComboBox2.ItemIndex := VideoGrabber.VideoSize;

   ComboBox3.Clear;
   ComboBox3.Items.Text := VideoGrabber.VideoCrossbarInputs;
   ComboBox3.ItemIndex := VideoGrabber.VideoCrossbarInput;

   Button4.Enabled := VideoGrabber.IsTVTunerAvailable;
end;
//------------------------------------------------------------------------------

procedure TForm1.OnFrameCaptureCompleted(Sender: TObject; const FrameData: TFrameData;
              DestType: TFrameCaptureDest; FileName: String; Success: Boolean);
begin
   Image1.Left := VideoGrabber.Left + VideoGrabber.Width + 10;
   Image1.AutoSize := true;
   Image1.Picture.Bitmap.Handle := FrameData.Bitmap.Handle;
   FrameData.Bitmap.ReleaseHandle;
end;
//------------------------------------------------------------------------------

procedure TForm1.OnResizeVideo(Sender: TObject; SourceWidth, SourceHeight: Integer);
var
   NewWidth: longint;
begin
   Image1.Picture.Assign (nil);
   Height := VideoGrabber.Top + VideoGrabber.Height + 80;
   Image1.Left := VideoGrabber.Left + VideoGrabber.Width + 20;
   Image1.Top := 55;
   Image1.Width := VideoGrabber.Width;
   Image1.Height := VideoGrabber.Height;
   NewWidth := Image1.Left + VideoGrabber.Width + 15;
   if NewWidth < GroupBox1.Width + 20 then begin
      NewWidth := GroupBox1.Width + 20;
   end;
   Width := NewWidth;
end;
//------------------------------------------------------------------------------

procedure TForm1.Button4Click(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.ShowDialog (dlg_TVTuner);
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.VideoDevice :=ComboBox1.ItemIndex;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.ComboBox2Change(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.VideoSize :=ComboBox2.ItemIndex;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.ComboBox3Change(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.VideoCrossbarInput :=ComboBox3.ItemIndex;
   end;
end;
//------------------------------------------------------------------------------

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
   if VideoGrabberCreated then begin
      VideoGrabber.Visible := CheckBox1.Checked;
   end;
end;
//------------------------------------------------------------------------------

end.
