unit MultiCams_;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, VidGrab, StdCtrls, ComCtrls, Buttons, FileCtrl;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Label3: TLabel;
    VideoGrabber1: TVideoGrabber;
    VideoGrabber2: TVideoGrabber;
    VideoGrabber3: TVideoGrabber;
    VideoGrabber4: TVideoGrabber;
    TabSheet2: TTabSheet;
    SpeedButton7: TSpeedButton;
    Panel1: TPanel;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Button1: TButton;
    Memo2: TMemo;
    RadioGroup1: TRadioGroup;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    CheckBox1: TCheckBox;
    Edit3: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure VideoGrabber1FrameCaptureCompleted(Sender: TObject;
      const FrameData: TFrameData; DestType: TFrameCaptureDest;
      FileName: String; Success: Boolean);
    procedure VideoGrabber2FrameCaptureCompleted(Sender: TObject;
      const FrameData: TFrameData; DestType: TFrameCaptureDest;
      FileName: String; Success: Boolean);
    procedure VideoGrabber3FrameCaptureCompleted(Sender: TObject;
      const FrameData: TFrameData; DestType: TFrameCaptureDest;
      FileName: String; Success: Boolean);
    procedure VideoGrabber4FrameCaptureCompleted(Sender: TObject;
      const FrameData: TFrameData; DestType: TFrameCaptureDest;
      FileName: String; Success: Boolean);
    procedure RadioGroup1Click(Sender: TObject);
    procedure SpeedButton7Click(Sender: TObject);
  private
    procedure SetupVideoCaptureDevice (VideoGrabber: TVideoGrabber; VideoDeviceIndex: longint);
    procedure ProcessFrameCapture (VideoGrabber: TVideoGrabber);
    procedure ProcessAutomaticCapture (VideoGrabber: TVideoGrabber);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
procedure TForm1.SetupVideoCaptureDevice (VideoGrabber: TVideoGrabber; VideoDeviceIndex: longint);
begin
   // NOTE: THE SETTINGS BELOW CAN BE DONE AT DESIGN TIME FROM THE OBJECT INSPECTOR

   // we assign each video capture device respectively to each component (0, 1, 2 and 3)
   VideoGrabber.VideoDevice := VideoDeviceIndex;

   // we assign a different prefix to file names for each camera during frame capture
   VideoGrabber.AutoFilePrefix := 'cam' + IntToStr (VideoDeviceIndex + 1) + '_';

   // if the video capture device is DV, this will divide the frame rate by 2 (15 fps NTSC or 12.5 fps PAL)
   VideoGrabber.DVReduceFrameRate := True;

   // otherwise we set 15 fps for non-DV video sources
   VideoGrabber.FrameRate := 15;

   // this will reduce the CPU load by dividing the preview frame rate by 2 (but this does not modify the capture frame rate
   VideoGrabber.ReducePreviewCPULoad := true;

   // we don't want to worry about sizes, we choose to preview in 320x240 using the nearest size available on the video capture device
   VideoGrabber.UseNearestVideoSize (320, 240, true);
end;
//------------------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
begin
   SetupVideoCaptureDevice (VideoGrabber1, 0);
   SetupVideoCaptureDevice (VideoGrabber2, 1);
   SetupVideoCaptureDevice (VideoGrabber3, 2);
   SetupVideoCaptureDevice (VideoGrabber4, 3);

   Edit3.Text := VideoGrabber1.StoragePath; // we use a common storage path in this project
end;
//------------------------------------------------------------------------------

procedure TForm1.ProcessFrameCapture (VideoGrabber: TVideoGrabber);
begin
   case RadioGroup1.ItemIndex of
      0: VideoGrabber.CaptureFrameTo (fc_TBitmap );
      1: VideoGrabber.CaptureFrameTo (fc_JpegFile);
      2: VideoGrabber.CaptureFrameTo (fc_BmpFile );
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.Button1Click(Sender: TObject);
begin
   CheckBox1.Checked := false; // stops the burst mode (see CheckBox1Click code)
   ProcessFrameCapture (VideoGrabber1);
   ProcessFrameCapture (VideoGrabber2);
   ProcessFrameCapture (VideoGrabber3);
   ProcessFrameCapture (VideoGrabber4);
end;
//---------------------------------------------------------------------------

procedure TForm1.ProcessAutomaticCapture (VideoGrabber: TVideoGrabber);
begin
   if CheckBox1.Checked then begin
      case RadioGroup1.ItemIndex of
         0: VideoGrabber.BurstType := fc_TBitmap ;
         1: VideoGrabber.BurstType := fc_JpegFile;
         2: VideoGrabber.BurstType := fc_BmpFile ;
      end;

      VideoGrabber.BurstCount := 0;    // don't stop capturing frames
      VideoGrabber.BurstMode := true;  // starts capturing frames automatically
      VideoGrabber.BurstInterval := StrToIntDef (Edit2.Text, 0); // we skip n frames for each frame captured
   end
   else begin
      VideoGrabber.BurstMode := false; // stops capturing frames automatically
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
   ProcessAutomaticCapture (VideoGrabber1);
   ProcessAutomaticCapture (VideoGrabber2);
   ProcessAutomaticCapture (VideoGrabber3);
   ProcessAutomaticCapture (VideoGrabber4);
end;
//---------------------------------------------------------------------------

procedure TForm1.VideoGrabber1FrameCaptureCompleted(Sender: TObject;
  const FrameData: TFrameData; DestType: TFrameCaptureDest;
  FileName: String; Success: Boolean);
begin
   Image1.Picture.Bitmap.Assign (FrameData.Bitmap);
   if DestType <> fc_TBitmap then begin
      Memo2.Lines.Add (FileName);
      Memo2.Lines.Add ('');
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.VideoGrabber2FrameCaptureCompleted(Sender: TObject;
  const FrameData: TFrameData; DestType: TFrameCaptureDest;
  FileName: String; Success: Boolean);
begin
   Image2.Picture.Bitmap.Assign (FrameData.Bitmap);
   if DestType <> fc_TBitmap then begin
      Memo2.Lines.Add (FileName);
      Memo2.Lines.Add ('');
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.VideoGrabber3FrameCaptureCompleted(Sender: TObject;
  const FrameData: TFrameData; DestType: TFrameCaptureDest;
  FileName: String; Success: Boolean);
begin
   Image3.Picture.Bitmap.Assign (FrameData.Bitmap);
   if DestType <> fc_TBitmap then begin
      Memo2.Lines.Add (FileName);
      Memo2.Lines.Add ('');
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.VideoGrabber4FrameCaptureCompleted(Sender: TObject;
  const FrameData: TFrameData; DestType: TFrameCaptureDest;
  FileName: String; Success: Boolean);
begin
   Image4.Picture.Bitmap.Assign (FrameData.Bitmap);
   if DestType <> fc_TBitmap then begin
      Memo2.Lines.Add (FileName);
      Memo2.Lines.Add ('');
   end;
end;
//---------------------------------------------------------------------------

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
   CheckBox1Click (CheckBox1); // to reflect the new value
end;
//---------------------------------------------------------------------------

procedure TForm1.SpeedButton7Click(Sender: TObject);
var
   Dir: String;
begin
   Dir := VideoGrabber1.StoragePath;

   if SelectDirectory (Dir, [sdAllowCreate, sdPerformCreate, sdPrompt], 0) then begin
      Edit3.Text := Dir;
      VideoGrabber1.StoragePath := Dir;
      VideoGrabber2.StoragePath := Dir;
      VideoGrabber3.StoragePath := Dir;
      VideoGrabber4.StoragePath := Dir;
   end;
end;
//---------------------------------------------------------------------------

end.
