unit Wizard;

interface

uses
  SysUtils, Classes, Controls, ExtCtrls, StdCtrls,
  Messages, ComCtrls, WizardTree, Graphics, Tabs;

type
  TWizard = class(TCustomControl)
  private
    //- Controls
    fControlPanel: TPanel;
    fPageControl: TPageControl;
    fNext: TButton;
    fPrev: TButton;
    fFinish: TButton;
    fHelp: TButton;
    fWizTree: TWizardTree;
    //- Captions
    fNextCaption: String;
    fPrevCaption: String;
    fFinishCaption: String;
    fHelpCaption: String;
    //- Events
    fOnNext: TNotifyEvent;
    fOnPrev: TNotifyEvent;
    fOnHelp: TNotifyEvent;
    fOnAbort: TNotifyEvent;
    fOnFinish: TNotifyEvent;
    fOnChanging: TTabChangingEvent;
    fOnChange: TNotifyEvent;
    //- Other
    fWizTreeWidth: LongInt;
    fTreeEnabled: Boolean;
    //-
    Procedure SetNextCaption( Caption: String );
    Procedure SetPrevCaption( Caption: String );
    Procedure SetHelpCaption( Caption: String );
    Procedure SetFinishCaption( Caption: String );
    Procedure SetPageControl( APageControl: TPageControl );
    Procedure DestroyControlPanel;
    Procedure ConstructControlPanel;
    Procedure NextClick( Sender: TObject );
    Procedure PrevClick( Sender: TObject );
    Procedure HelpClick( Sender: TObject );
    Procedure FinishClick( Sender: TObject );
    Procedure WizardClick( Sender: TObject );
    Function GetHelpEnabled: Boolean;
    Procedure SetHelpEnabled( AVal: Boolean );
    Procedure SetWizTreeWidth( aWidth: LongInt );
    Function GetTreeColor: TColor;
    Function GetBoxColor: TColor;
    Function GetSBoxColor: TColor;
    Function GetLineColor: TColor;
    Procedure SetTreeColor( AColor: TColor );
    Procedure SetBoxColor( AColor: TColor );
    Procedure SetSBoxColor( AColor: TColor );
    Procedure SetLineColor( AColor: TColor );
    Procedure GotoPage( Page: LongInt );
    Function GetActivePage: LongInt;
    Procedure SetTreeEnabled( ABool: Boolean );
  protected
    Procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  public
    Constructor Create( aOwner: TComponent ); override;
    Destructor Destroy; override;

  published
    //-The page control
    Property TreeEnabled: Boolean read fTreeEnabled write SetTreeEnabled;
    Property PageControl: TPageControl read fPageControl write SetPageControl;
    Property TreeWidth: LongInt read fWizTreeWidth write SetWizTreeWidth;
    Property TreeColor: TColor read GetTreeColor write SetTreeColor;
    Property TreeBoxColor: TColor read GetBoxColor Write SetBoxColor;
    Property TreeSelectedBoxColor: TColor read GetSBoxColor Write SetSBoxColor;
    Property TreeLineColor: TColor read GetLineColor write SetLineColor;
    Property ActivePage: LongInt read GetActivePage write GotoPage;


    //-Is help available?
    Property HelpEnabled: Boolean read GetHelpEnabled write SetHelpEnabled;

    //- Control panel button captions
    Property NextCaption: String read fNextCaption write SetNextCaption;
    Property PrevCaption: String read fPrevCaption write SetPrevCaption;
    Property FinishCaption: String read fFinishCaption write SetFinishCaption;
    Property HelpCaption: String read fHelpCaption write SetHelpCaption;

    //- Events
    Property OnNext: TNotifyEvent read fOnNext write fOnNext;
    Property OnPrev: TNotifyEvent read fOnPrev write fOnPrev;
    Property OnAbort: TNotifyEvent read fOnAbort write fOnAbort;
    Property OnFinish: TNotifyEvent read fOnFinish write fOnFinish;
    Property OnHelp: TNotifyEvent read fOnHelp write fOnHelp;
    Property OnChanging: TTabChangingEvent read fOnChanging write fOnChanging;
    Property OnChange: TNotifyEvent read fOnChange write fOnChange;
  end;

procedure Register;

implementation

Procedure TWizard.SetTreeEnabled( ABool: Boolean );
Begin
//-
fTreeEnabled:=ABool;
fWizTree.Visible:=ABool;
if Width>399 then
   Begin
   fControlPanel.Width:=Width;
   fControlPanel.Align:=alBottom;
   fHelp.Left:=8;
   fHelp.Top:=8;
   fFinish.Left:=Width-80;
   fFinish.Top:=8;
   fNext.Left:=Width-168;
   fNext.Top:=8;
   fPrev.Left:=Width-248;
   fPrev.Top:=8;
   fWizTree.Top:=0;
   fWizTree.Left:=0;
   fWizTree.Height:=Height-fControlPanel.Height;
   fWizTree.Width:=fWizTreeWidth;
   if assigned(fPageControl) then
      Begin
      fPageControl.Top:=0;
      if fTreeEnabled then fPageControl.Left:=fWizTreeWidth
                      else fPageControl.Left:=0;
      if fTreeEnabled then fPageControl.Width:=Width-fWizTreeWidth
                      else fPageControl.Width:=Width;
      fPageControl.Height:=Height-15;
      End;
   End;
End;


Function TWizard.GetActivePage: LongInt;
Begin
Result:=fWizTree.ItemIndex;
End;

Function TWizard.GetTreeColor: TColor;
Begin
Result:=fWizTree.Color;
End;

Function TWizard.GetBoxColor: TColor;
Begin
Result:=fWizTree.BoxColor;
End;

Function TWizard.GetSBoxColor: TColor;
Begin
Result:=fWizTree.SelectedBoxColor;
End;

Function TWizard.GetLineColor: TColor;
Begin
Result:=fWizTree.LineColor;
End;

Procedure TWizard.SetTreeColor( AColor: TColor );
Begin
fWizTree.Color:=AColor;
End;

Procedure TWizard.SetBoxColor( AColor: TColor );
Begin
fWizTree.BoxColor:=AColor;
End;

Procedure TWizard.SetSBoxColor( AColor: TColor );
Begin
fwizTree.SelectedBoxColor:=AColor;
End;

Procedure TWizard.SetLineColor( AColor: TColor );
Begin
fWizTree.LineColor:=AColor;
End;


Procedure TWizard.SetWizTreeWidth( aWidth: LongInt );
Begin
//-
fWizTree.Width:=aWidth;
fWizTreeWidth:=fWizTree.Width;
if assigned(fPageControl) then
   Begin
   fPageControl.Left:=fWizTreeWidth;
   fPageControl.Width:=Width-fWizTreeWidth;
   fPageControl.Height:=Height-15;
   End;
End;

procedure TWizard.Notification(AComponent :TComponent; Operation : TOperation);
begin
  inherited; //Never forget to call this
  if (Operation = opRemove) and
     (AComponent = fPageControl) then
      fPageControl:=nil;
end;

Procedure TWizard.NextClick( Sender: TObject );
Begin
if Assigned(fPageControl) then
   Begin
   GotoPage( fPageControl.TabIndex+1 );
   if assigned(fOnNext) then fOnNext( Sender );
   End;
End;

Function TWizard.GetHelpEnabled: Boolean;
Begin
Result:=fHelp.Visible;
End;

Procedure TWizard.SetHelpEnabled( AVal: Boolean );
Begin
fHelp.Visible:=AVal;
End;


Procedure TWizard.PrevClick( Sender: TObject );
Begin
if assigned( fPageControl ) then
   Begin
   GotoPage( fPageControl.TabIndex-1 );
   if assigned(fOnPrev) then fOnPrev( Sender );
   End;
End;

Procedure TWizard.GotoPage( Page: LongInt );
Var Allow: Boolean;
Begin
Allow:=True;
if assigned(fOnChanging) then fOnChanging(Self,Allow);
if (assigned( fPageControl )) AND (Page>-1) AND
   (Page<fPageControl.PageCount) AND (Allow) then
     Begin
     fPageControl.TabIndex:=Page;
     fWizTree.ItemIndex:=Page;
     if fPageControl.TabIndex=0 then
        fPrev.Visible:=False else fPrev.Visible:=True;
     if fPageControl.TabIndex=fPageControl.PageCount-1 then
        fNext.Visible:=False else fNext.Visible:=True;
     if assigned(fOnChange) then fOnChange(Self);
     End;
End;

Procedure TWizard.WizardClick( Sender: TObject );
Begin
//- Wizard tree has been clicked.
GotoPage( fWizTree.ItemIndex );
End;

Procedure TWizard.HelpClick( Sender: TObject );
Begin
if assigned(fOnHelp) then fOnHelp( Sender );
End;

Procedure TWizard.FinishClick( Sender: TObject );
Begin
If Assigned( fPageControl ) then
   Begin
   if fPageControl.TabIndex=fPageControl.PageCount-1 then
      Begin
      if assigned(fOnFinish) then fOnFinish(Sender);
      End Else
      Begin
      if assigned(fOnAbort) then fOnAbort(Sender);
      End;
   End;
End;


Procedure TWizard.ConstructControlPanel;
Begin
 //- Create the control panel
 fControlPanel:=TPanel.Create( Self );
 fControlPanel.Parent:=Self;
 fControlPanel.Width:=Width;
 fControlPanel.Align:=alBottom;
 //- Create the Help Button
 fHelp:=TButton.Create( fControlPanel );
 fHelp.Parent:=fControlPanel;
 fHelp.Left:=8;
 fHelp.Top:=8;
 fHelp.Caption:=fHelpCaption;
 fHelp.OnClick:=HelpClick;
 //- Create the finish button
 fFinish:=TButton.Create( fControlPanel );
 fFinish.Parent:=fControlPanel;
 fFinish.Left:=Width-80;
 fFinish.Top:=8;
 fFinish.Caption:=fFinishCaption;
 fFinish.OnClick:=FinishClick;
 //- Create the next button
 fNext:=TButton.Create( fControlPanel );
 fNext.Parent:=fControlPanel;
 fNext.Left:=Width-168;
 fNext.Top:=8;
 fNext.Caption:=fNextCaption;
 fNext.OnClick:=NextClick;
 //- Create the prev button
 fPrev:=TButton.Create( fControlPanel );
 fPrev.Parent:=fControlPanel;
 fPrev.Left:=Width-248;
 fPrev.Top:=8;
 fPrev.Caption:=fPrevCaption;
 fPrev.OnClick:=PrevClick;
End;

Procedure TWizard.DestroyControlPanel;
Begin
 fHelp.Destroy;
 fNext.Destroy;
 fPrev.Destroy;
 fFinish.Destroy;
 fControlPanel.Destroy;
End;

Constructor TWizard.Create( aOwner: TComponent );
Begin
 inherited Create( aOwner );
 Width:=400;
 Height:=400;
 //- Initialize captions
 fHelpCaption:='Help';
 fNextCaption:='Next >>';
 fPrevCaption:='<< Prev';
 fFinishCaption:='Finish';
 ConstructControlPanel;
 fWizTreeWidth:=100;
 fWizTree:=TWizardTree.Create( Self );
 fWizTree.Parent:=Self;
 fWizTree.Top:=0;
 fWizTree.Left:=0;
 fWizTree.Width:=fWizTreeWidth;
 fWizTree.Height:=Height-fControlPanel.Height;
 fWizTree.Items.Clear;
 fWizTree.OnChange:=WizardClick;
 fTreeEnabled:=True;
 fWizTree.Visible:=True;
 GotoPage(0);
End;

Destructor TWizard.Destroy;
Begin
 if Assigned(fPageControl) then fPageControl.Destroy;
 DestroyControlPanel;
 fWizTree.Destroy;
 inherited Destroy;
End;

Procedure TWizard.SetPageControl(aPageControl: TPageControl);
Var x: LongInt;
Begin
if assigned(fPageControl) then fPageControl.Parent:=Self.Parent;
//-Assign the new control
fPageControl:=aPageControl;
//-Setup the new control, in design mode the tabs are visible.
//-otherwize the control panel is reconstructed to hide them.
if assigned(fPageControl) then
   Begin
   fPageControl.Align:=alCustom;
   fPageControl.Parent:=Self;
   fPageControl.Top:=0;
   fPageControl.Left:=fWizTreeWidth;
   fPageControl.Width:=Width-fWizTreeWidth;
   fPageControl.Height:=Height-15;
   fPageControl.TabPosition:=tpTop;
   fPageControl.Style:=tsTabs;
   fPageControl.TabPosition:=tpBottom;
   //-
   fWizTree.Items.Clear;
   for x:=0 to fPageControl.PageCount-1 do
       fWizTree.Items.Add(fPageControl.Pages[X].Caption);
   fWizTree.Repaint;
   //- Now cover the tabs.
   DestroyControlPanel;
   ConstructControlPanel;
   GotoPage(0);
   End;
End;


Procedure TWizard.SetNextCaption( Caption: String );
Begin
fNextCaption:=Caption;
fNext.Caption:=Caption;
End;

Procedure TWizard.SetPrevCaption( Caption: String );
Begin
fPrevCaption:=Caption;
fPrev.Caption:=Caption;
End;

Procedure TWizard.SetHelpCaption( Caption: String );
Begin
fHelpCaption:=Caption;
fHelp.Caption:=Caption;
End;

Procedure TWizard.SetFinishCaption( Caption: String );
Begin
fFinishCaption:=Caption;
fFinish.Caption:=Caption;
End;


procedure TWizard.WMSize(var Message: TWMSize);
begin
if Width>399 then
   Begin
   fControlPanel.Width:=Width;
   fControlPanel.Align:=alBottom;
   fHelp.Left:=8;
   fHelp.Top:=8;
   fFinish.Left:=Width-80;
   fFinish.Top:=8;
   fNext.Left:=Width-168;
   fNext.Top:=8;
   fPrev.Left:=Width-248;
   fPrev.Top:=8;
   fWizTree.Top:=0;
   fWizTree.Left:=0;
   fWizTree.Height:=Height-fControlPanel.Height;
   fWizTree.Width:=fWizTreeWidth;
   if assigned(fPageControl) then
      Begin
      fPageControl.Top:=0;
      if fTreeEnabled then fPageControl.Left:=fWizTreeWidth
                      else fPageControl.Left:=0;
      if fTreeEnabled then fPageControl.Width:=Width-fWizTreeWidth
                      else fPageControl.Width:=Width;
      fPageControl.Height:=Height-15;
      End;
   End;
end;



procedure Register;
begin
  RegisterComponents('Nation', [TWizard]);
end;

end.
